内包表記

# 目的

内包表記について説明する。とみせかけて気持ちモナドの話をする。
ついでにSQLも絡める

# 対象

内包表記をよく知らない人

# 内容

## 内包表記とは 5分

内包表記って機能があるらしい言語

[(x,y) | x <- [1..9], y <- [1..9], x * y == 24]
[(x,y,z) | x <- [1..9], y <- [1..9], z <- [1..9], x * y * z == 24]
[(x,y)
    for x in range(1,10)
    for y in range(1,10)
    if x * y == 24
]

[(x,y) | x <- [1..9],
  y <- [1..9],
  x * y == 24]

* Python
* Erlang
* Scala
* JavaScript 1.7
* Haskell

集合の記述方法には

* 外延的記法
* 内包的記法

がある

外延的記法 - 具体的にどんなものがあるのか
内包的記法 - 性質で表現する

具体例

外延 { 1,3,5,7,9 }
内包 { x | x は 10未満の正の奇数 }
    { x | x ∈ ℕ, x < 10, x % 2 = 1 }
プログラミング言語でリストを記述する方法

Haskell

外延 [1,3,5,7,9]
内包 [ x | x <- [1..9], x `mod` 2 == 1 ]

集合の内包的
    { x | x ∈ {1,2,3,4,5,6,7,8,9}, x % 2 == 1 }

リストの内包表記
   list comprehension
内包的記法
   intensional definition

内包表記

### 内包表記の便利なところ？ 3分

リストの内包表記 (list comprehension) は、 map() や filter() や lambda を使わずにリストを生成するための簡潔な方法を提供しています。
内包表記によるリストの定義は、たいてい map(), filter(), lambda を使ってリストを生成するコードよりも明快になります。

```
[*1..9].map do |x|
  [*1..9].map do |y|
    [x,y]
  end
end.flatten(1).select do |x,y|
  x * y == 24
end
```

```
[*1..9].map do |x|
  [*1..9].map do |y|
    [*1..9].map do |z|
      [x,y,z]
    end
  end
end.flatten(2).select do |x,y,z|
  x * y * z == 24
end
```

```
[*1..9].
  product([*1..9],[*1..9]).
  select do |x,y,z|
    x * y * z == 24
  end
```

解 が 32 とか 24 になる積を探す


### 内包表記とSQL 3分

そういえば内包表記って何か似てない?

```
SELECT x,y
FROM generate_series(1,9) AS X,
     generate_series(1,9) AS Y
WHERE x * y = 24;
```

```
SELECT x,y,z
FROM generate_series(1,9) AS X,
     generate_series(1,9) AS Y,
     generate_series(1,9) AS Z
WHERE x * y * z = 24;
```

集合…データベース!!

SELECT と FROM と WHERE

内包表記で group とか order とか

http://www.kotha.net/ghcguide_ja/7.6.2/syntax-extns.html#generalised-list-comprehensions

```
:set -XTransformListComp
[ (x,y) | x <- [1..9],
        y <- [1..9],
        x * y == 24,
        then take 2]
```

```
:set -XTransformListComp
:m GHC.Exts
[ (x,y) | x <- [1..9],
        y <- [1..9],
        x * y == 24,
        then sortWith by y]
```

## そういえばモナド 3分

do 構文をつかっても内包表記と同じことができる。


```
do
  x <- [1..9]
  y <- [1..9]
  guard (x * y == 24)
  return (x,y)
```

そっくり

リストじゃなくてもいいんじゃね？モナドであれば内包表記が使える

Maybe モナドの内包表記

```
:set -XMonadComprehensions
[ (x,y) | x <- Just 3,
        y <- Just 8,
        x * y == 24]
```

## まとめ

外延表記 - 全部並べる
内包表記 - 意味をリストを表現する
SQLに似てるらしい
リスト以外の内包表記もある
モナドこわくななそう

# ツイート

https://twitter.com/eielh/status/517681959837761536
何もなかったちっぽけな自分が歩いた道にたくさんの成果物があって、気がついたら何かある自分になってた。

https://twitter.com/eielh/status/517304159792422914
聞いた言葉を真っ直ぐ受け止めて、真っ直ぐ返せるようになろうとおもった。

https://twitter.com/eielh/status/527097278641614848
よくあるく道も、時間や天気によって違った景色を見せる。

https://twitter.com/eielh/status/518973046606929921
格好をつけずに考えたこと、思ったこと記録していこう。ダメなのところを治す努力もしながら。


# 参考文献

* [内包と外延 - Wikipedia](http://ja.wikipedia.org/wiki/%E5%86%85%E5%8C%85%E3%81%A8%E5%A4%96%E5%BB%B6)
* [内包的記法の出展 - 集合 - Wikipedia](http://ja.wikipedia.org/wiki/%E9%9B%86%E5%90%88)
* intentional - 意図的な http://eow.alc.co.jp/search?q=intentional&ref=sa
* [5. データ構造 — Python 2.7ja1 documentation](http://docs.python.jp/2/tutorial/datastructures.html#id6)
* [7.3. 構文的拡張 - 内包表記の拡張](http://www.kotha.net/ghcguide_ja/7.6.2/syntax-extns.html#generalised-list-comprehensions)
* [7.3. 構文的拡張 Monadの内包表記](http://www.kotha.net/ghcguide_ja/7.6.2/syntax-extns.html#monad-comprehensions)
[Haskell - ghciで言語拡張を有効にする - Qiita](http://qiita.com/uduki845/items/d60dc51ad3a26b9ab430)

* 中奥くん
* もりしー
